'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _reactRouter = require('react-router');

var _RelativeLinks = require('./RelativeLinks');

var _reactDom = require('react-dom');

var _createMemoryHistory = require('react-router/lib/createMemoryHistory');

var _createMemoryHistory2 = _interopRequireDefault(_createMemoryHistory);

var _reactRouterApplyMiddleware = require('react-router-apply-middleware');

var _reactRouterApplyMiddleware2 = _interopRequireDefault(_reactRouterApplyMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('RelativeLink', function () {

  describe('without params', function () {

    var run = function run(link, initialPath, assertion) {
      var linkWithId = _react2.default.cloneElement(link, { className: '__subject__' });

      var Container = _react2.default.createClass({
        displayName: 'Container',
        componentDidMount: function componentDidMount() {
          assertion(div.querySelector('a.__subject__'));
        },
        render: function render() {
          return this.props.children;
        }
      });

      var Parent = function Parent(_ref) {
        var children = _ref.children;
        return _react2.default.createElement(
          'div',
          null,
          'Parent ',
          children
        );
      };
      var Reference = function Reference(_ref2) {
        var children = _ref2.children;
        return _react2.default.createElement(
          'div',
          null,
          'Reference ',
          linkWithId,
          ' ',
          children
        );
      };
      var Child = function Child() {
        return _react2.default.createElement(
          'div',
          null,
          'Child'
        );
      };
      var Sibling = function Sibling(_ref3) {
        var children = _ref3.children;
        return _react2.default.createElement(
          'div',
          null,
          'Sibling ',
          children
        );
      };
      var Neice = function Neice() {
        return _react2.default.createElement(
          'div',
          null,
          'Neice'
        );
      };

      var routes = _react2.default.createElement(
        _reactRouter.Route,
        { path: '/', component: Container },
        _react2.default.createElement(
          _reactRouter.Route,
          { path: 'parent', component: Parent },
          _react2.default.createElement(
            _reactRouter.Route,
            { path: 'reference', component: Reference },
            _react2.default.createElement(_reactRouter.Route, { path: 'child', component: Child })
          ),
          _react2.default.createElement(
            _reactRouter.Route,
            { path: 'sibling', component: Sibling },
            _react2.default.createElement(_reactRouter.Route, { path: 'neice', component: Neice })
          )
        )
      );

      var div = document.createElement('div');

      (0, _reactDom.render)(_react2.default.createElement(_reactRouter.Router, {
        render: (0, _reactRouterApplyMiddleware2.default)((0, _RelativeLinks.useRelativeLinks)()),
        routes: routes,
        history: (0, _createMemoryHistory2.default)(initialPath)
      }), div);
    };

    var describeLinks = function describeLinks(startPath) {
      return function () {
        it('links to parent', function (done) {
          run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: '..' }), startPath, function (node) {
            (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent');
            done();
          });
        });

        it('links to self', function (done) {
          run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: '.' }), startPath, function (node) {
            (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/reference');
            done();
          });
        });

        it('links to children', function (done) {
          run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: './child' }), startPath, function (node) {
            (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/reference/child');
            done();
          });
        });

        it('links to siblings', function (done) {
          run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: '../sibling' }), startPath, function (node) {
            (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/sibling');
            done();
          });
        });

        it('links to neice', function (done) {
          run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: '../sibling/neice' }), startPath, function (node) {
            (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/sibling/neice');
            done();
          });
        });

        describe('with a location descriptor', function () {
          it('works', function (done) {
            var loc = { pathname: '.' };
            run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: loc }), startPath, function (node) {
              (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/reference');
              done();
            });
          });

          it('works with a query', function (done) {
            var loc = { pathname: '.', query: { foo: 'bar' } };
            run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: loc }), startPath, function (node) {
              (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/reference?foo=bar');
              done();
            });
          });

          it('works with only a query', function (done) {
            var loc = { query: { foo: 'bar' } };
            run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: loc }), startPath, function (node) {
              (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/reference?foo=bar');
              done();
            });
          });
        });
      };
    };

    describe('when rendered at a terminal route', describeLinks('/parent/reference'));
    describe('when rendered in a parent route', describeLinks('/parent/reference/child'));
  });

  describe('with params', function () {
    var run = function run(link, initialPath, assertion) {
      var linkWithId = _react2.default.cloneElement(link, { className: '__subject__' });

      var Container = _react2.default.createClass({
        displayName: 'Container',
        componentDidMount: function componentDidMount() {
          assertion(div.querySelector('a.__subject__'));
        },
        render: function render() {
          return this.props.children;
        }
      });

      var Parent = function Parent(_ref4) {
        var children = _ref4.children;
        return _react2.default.createElement(
          'div',
          null,
          'Parent ',
          children
        );
      };
      var Reference = function Reference(_ref5) {
        var children = _ref5.children;
        return _react2.default.createElement(
          'div',
          null,
          'Reference ',
          linkWithId,
          ' ',
          children
        );
      };
      var Child = function Child() {
        return _react2.default.createElement(
          'div',
          null,
          'Child'
        );
      };
      var Sibling = function Sibling(_ref6) {
        var children = _ref6.children;
        return _react2.default.createElement(
          'div',
          null,
          'Sibling ',
          children
        );
      };
      var Neice = function Neice() {
        return _react2.default.createElement(
          'div',
          null,
          'Neice'
        );
      };

      var routes = _react2.default.createElement(
        _reactRouter.Route,
        { path: '/', component: Container },
        _react2.default.createElement(
          _reactRouter.Route,
          { path: 'parent/:p', component: Parent },
          _react2.default.createElement(
            _reactRouter.Route,
            { path: 'reference/:r', component: Reference },
            _react2.default.createElement(_reactRouter.Route, { path: 'child/:c', component: Child })
          ),
          _react2.default.createElement(
            _reactRouter.Route,
            { path: 'sibling/:s', component: Sibling },
            _react2.default.createElement(_reactRouter.Route, { path: 'neice/:n', component: Neice })
          )
        )
      );

      var div = document.createElement('div');

      (0, _reactDom.render)(_react2.default.createElement(_reactRouter.Router, {
        render: (0, _reactRouterApplyMiddleware2.default)((0, _RelativeLinks.useRelativeLinks)()),
        routes: routes,
        history: (0, _createMemoryHistory2.default)(initialPath)
      }), div);
    };

    var describeLinks = function describeLinks(startPath) {
      return function () {
        it('links to parent', function (done) {
          run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: '../../../a' }), startPath, function (node) {
            (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/a');
            done();
          });
        });

        it('links to self', function (done) {
          run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: '.' }), startPath, function (node) {
            (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/1/reference/2');
            done();
          });
        });

        it('links to self with new params', function (done) {
          run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: '../a' }), startPath, function (node) {
            (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/1/reference/a');
            done();
          });
        });

        it('links to children', function (done) {
          run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: './child/a' }), startPath, function (node) {
            (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/1/reference/2/child/a');
            done();
          });
        });

        it('links to siblings', function (done) {
          run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: '../../sibling/a' }), startPath, function (node) {
            (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/1/sibling/a');
            done();
          });
        });

        it('links to neice', function (done) {
          run(_react2.default.createElement(_RelativeLinks.RelativeLink, { to: '../../sibling/a/neice/b' }), startPath, function (node) {
            (0, _expect2.default)(node.getAttribute('href')).toEqual('/parent/1/sibling/a/neice/b');
            done();
          });
        });
      };
    };

    describe('when rendered at a terminal route', describeLinks('/parent/1/reference/2'));
    describe('when rendered in a parent route', describeLinks('/parent/1/reference/2/child/3'));
  });
}); /*eslint-env mocha*/