'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RelativeLink = exports.RelativeLinksContainer = exports.createElement = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _Link = require('react-router/lib/Link');

var _Link2 = _interopRequireDefault(_Link);

var _PatternUtils = require('react-router/lib/PatternUtils');

var _resolvePathname = require('resolve-pathname');

var _resolvePathname2 = _interopRequireDefault(_resolvePathname);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// 1. Use this in `<Router createElement={RelativeLinks.createElement}/>`
var createElement = exports.createElement = function createElement(Component, props) {
  return _react2.default.createElement(RelativeLinksContainer, { Component: Component, routerProps: props });
};

var _React$PropTypes = _react2.default.PropTypes;
var oneOfType = _React$PropTypes.oneOfType;
var shape = _React$PropTypes.shape;
var object = _React$PropTypes.object;
var string = _React$PropTypes.string;
var func = _React$PropTypes.func;
var array = _React$PropTypes.array;


var relativeLinksContextType = {
  relativeLinks: shape({
    params: object.isRequired,
    route: object.isRequired,
    routes: array.isRequired
  }).isRequired
};

// 2. the library renders route components with some extra context
//    particularly, the route the component that it's assigned to
//    and params
var RelativeLinksContainer = exports.RelativeLinksContainer = _react2.default.createClass({
  displayName: 'RelativeLinksContainer',


  propTypes: {
    Component: func.isRequired,
    routerProps: shape({
      route: object.isRequired,
      params: object.isRequired
    }).isRequired
  },

  getDefaultProps: function getDefaultProps() {
    /* Normal usage is:
    ```js
    <Router createElement={relativeLinksCreateElement}/>
    ```
     This goofy prop lets "createElement" abstractions compose like:
     ```js
    <Router createElement={(Component, props) => (
      <AsyncPropsContainer
        Component={Component}
        routerProps={props}
        createElement={(Component, props) => (
          <RelativeLinksContainer Component={Component} routerProps={props}/>
        )}
      />
    )}/>
    ```
    */
    return {
      createElement: function createElement(Component, props) {
        return _react2.default.createElement(Component, props);
      }
    };
  },


  childContextTypes: relativeLinksContextType,

  getChildContext: function getChildContext() {
    var _props$routerProps = this.props.routerProps;
    var params = _props$routerProps.params;
    var routes = _props$routerProps.routes;
    var route = _props$routerProps.route;

    return { relativeLinks: { params: params, route: route, routes: routes } };
  },
  render: function render() {
    return this.props.createElement(this.props.Component, this.props.routerProps);
  }
});

var isAbsolute = function isAbsolute(to) {
  return to.match(/^\//);
};

var constructRoutePattern = function constructRoutePattern(route, routes) {
  var paths = [];

  // we need to return a path that starts with `/`, at least one route
  // in the matched routes will start with `/`, once we find it, we can
  // just `join` the rest, when approaching other ways I kept running into
  // issues where it returned //some/path instead of /some/path ...
  var rootPath = null;

  for (var i = 0, l = routes.length; i < l; i++) {
    if (routes[i] === route) {
      // we're done, we made it to our route, the rest are irrelevant
      break;
    } else if (isAbsolute(routes[i].path)) {
      if (rootPath) {
        // this will throw even though they may have a valid route config like:
        //
        // <Route path="/">
        //   <Route path="/foo"/>
        // </Route>
        //
        // but I'm just punting for now
        throw new Error('You can\'t use relative links in route configs with nested absolute route paths');
      } else {
        rootPath = routes[i].path;
      }
    } else if (routes[i].path) {
      paths.push(routes[i].path);
    }
  }

  if (route.path) paths.push(route.path);

  var childPath = paths.join('/');
  return rootPath === '/' ? '/' + childPath : rootPath + '/' + childPath;
};

var resolvePathname = function resolvePathname(_ref) {
  var relativePath = _ref.relativePath;
  var route = _ref.route;
  var routes = _ref.routes;
  var params = _ref.params;

  var patternUpToRoute = constructRoutePattern(route, routes);
  // TODO: remove trailing slash hack
  // we add a slash cause it's SUPER WEIRD if we don't, should add an option
  // to history to always use trailing slashes to not do this and cause
  // confusion for people who actually know how browsers resolve urls :P
  var specialCase = relativePath.trim() === '';
  var slash = specialCase ? '' : '/';
  var resolvedPattern = (0, _resolvePathname2.default)('' + relativePath + slash, patternUpToRoute + '/');
  var withoutSlash = resolvedPattern.substring(0, resolvedPattern.length - 1);
  return (0, _PatternUtils.formatPattern)(withoutSlash, params);
};

// 3. Use a `<RelativeLink to="somewhere/relative"/>`
var RelativeLink = exports.RelativeLink = _react2.default.createClass({
  displayName: 'RelativeLink',


  propTypes: {
    to: oneOfType([string, object]).isRequired
  },

  contextTypes: relativeLinksContextType,

  getInitialState: function getInitialState() {
    // gonna cache this for the perfs
    return { to: this.calculateTo(this.props) };
  },
  componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
    if (nextProps.to !== this.props.to) {
      this.setState({ to: this.calculateTo(nextProps) });
    }
  },
  calculateTo: function calculateTo(props) {
    var to = props.to;
    var _context$relativeLink = this.context.relativeLinks;
    var route = _context$relativeLink.route;
    var routes = _context$relativeLink.routes;
    var params = _context$relativeLink.params;

    var isLocationDescriptor = (typeof to === 'undefined' ? 'undefined' : _typeof(to)) === 'object';
    var relativePath = isLocationDescriptor ? to.pathname || '' : to;
    var resolved = isAbsolute(relativePath) ? relativePath : resolvePathname({ relativePath: relativePath, route: route, routes: routes, params: params });
    return isLocationDescriptor ? _extends({}, to, { pathname: resolved }) : resolved;
  },
  render: function render() {
    return _react2.default.createElement(_Link2.default, _extends({}, this.props, { to: this.state.to }));
  }
});